using System;
using System.Collections;
using System.Drawing;
using System.Reflection;
using System.IO;

using org.zefer.pd4ml;

namespace PD4Converter
{
    class Tokenizer
    {
        private System.Collections.ArrayList elements;
        private string source;
        private string delimiters = " \t\n\r";      

        public Tokenizer(string source)
        {           
            this.elements = new System.Collections.ArrayList();
            this.elements.AddRange(source.Split(this.delimiters.ToCharArray()));
            this.RemoveEmptyStrings();
            this.source = source;
        }

        public Tokenizer(string source, string delimiters)
        {
            this.elements = new System.Collections.ArrayList();
            this.delimiters = delimiters;
            this.elements.AddRange(source.Split(this.delimiters.ToCharArray()));
            this.RemoveEmptyStrings();
            this.source = source;
        }

        public int Count
        {
            get
            {
                return (this.elements.Count);
            }
        }

        public bool HasMoreTokens()
        {
            return (this.elements.Count > 0);           
        }

        public string NextToken()
        {           
            string result;
            if (source == "") throw new System.Exception();
            else
            {
                this.elements = new System.Collections.ArrayList();
                this.elements.AddRange(this.source.Split(delimiters.ToCharArray()));
                RemoveEmptyStrings();       
                result = (string) this.elements[0];
                this.elements.RemoveAt(0);              
                this.source = this.source.Remove(this.source.IndexOf(result),result.Length);
                this.source = this.source.TrimStart(this.delimiters.ToCharArray());
                return result;                  
            }           
        }

        public string NextToken(string delimiters)
        {
            this.delimiters = delimiters;
            return NextToken();
        }

        private void RemoveEmptyStrings()
        {
            for (int index=0; index < this.elements.Count; index++)
                if ((string)this.elements[index]== "")
                {
                    this.elements.RemoveAt(index);
                    index--;
                }
        }
    }

    /// <summary>
    /// Summary description for Class1.
    /// </summary>
    public class PD4Converter
    {
        public void generatePDF(String inputUrl, int htmlWidth, String pageFormat, int permissions, 
                String bookmarks, String orientation, String insets, String fontsDir, 
                Stream outfile)
        {
            System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("en-US");
            System.Threading.Thread.CurrentThread.CurrentUICulture = System.Threading.Thread.CurrentThread.CurrentCulture;

            PD4ML pd4ml = new PD4ML();
            if ( insets != null ) {
                Tokenizer st = new Tokenizer( insets, ",");
                try {
                    int top = Int32.Parse(st.NextToken());
                    int left = Int32.Parse(st.NextToken());
                    int bottom = Int32.Parse(st.NextToken());
                    int right = Int32.Parse(st.NextToken());
                    String units = st.NextToken();
                    Rectangle ins = new Rectangle(top, left, bottom, right);
                    if ("MM".Equals(units.ToUpper())) {
                        pd4ml.PageInsetsMM = (ins);
                    } else {
                        pd4ml.PageInsets = (ins);
                    }
                } catch (Exception e) {
                    throw new Exception(
                            "Invalid page insets (top, left, bottom, right, units): "
                                    + insets );
                }
            }
            
            pd4ml.HtmlWidth = (htmlWidth);
            if(orientation == null) orientation = "portrait";
            if ("PORTRAIT".Equals(orientation.ToUpper())) 
                pd4ml.PageSize = PD4Constants.getSizeByName(pageFormat);
            else 
                pd4ml.PageSize = pd4ml.changePageOrientation( PD4Constants.getSizeByName(pageFormat) ); 
            
            if ( permissions != -1 ) pd4ml.setPermissions("empty", permissions, true);
            if ( bookmarks != null ) {
                if ( "ANCHORS".Equals(bookmarks.ToUpper()) )
                    pd4ml.generateOutlines(false);
                else if ( "HEADINGS".Equals(bookmarks.ToUpper()) ) 
                    pd4ml.generateOutlines(true);
            }

            if ( fontsDir != null && fontsDir.Length > 0 ) pd4ml.useTTF( fontsDir, true );

            pd4ml.render(new Uri(inputUrl), outfile);
        }
    }
}
